#include "Fusion_SLAM_utils.h"
#include <math.h>

#define RADIUS_INIT_VALUE (999999.F) /* in m */
#define CURVATURE_USE_CIRCLE_EQUATION (1.F/1000.F) /* in 1/m */
#define C_HALFPI (0.5F * 3.14159265359f)

float32 BML_f_Abs(float32 x){
    return (x < 0.f) ? -x : x;
}

boolean BML_f_IsZero(float32 x){
    return (BML_f_Abs(x) < 1e-12) ? TRUE : FALSE;
}

float32 SQR(float32 x){
    return (x * x);
}

float32 SQRT_(float32 x){
    return sqrtf(x);
}

float32 ATAN2_(float32 x, float32 y){
    return atan2f(x, y);
}

uint32 BML_u_Round2UintGen(float32 x){
		return (uint32)(x + 0.5f);
}

BML_t_TrajRefPoint CML_CalculateDistancePoint2Circle(float32 f_X, float32 f_Y, float32 f_C0){
    float32 f_Radius         = RADIUS_INIT_VALUE;
    float32 f_R              = 0.f;
    float32 f_DistToCourse   = 0.f;
    float32 f_DistOnCourse   = 0.f;
    float32 f_NormVecX       = 0.f;
    float32 f_NormVecY       = 0.f;
    float32 f_RefCourseDistX = 0.f;
    float32 f_RefCourseDistY = 0.f;
    BML_t_TrajRefPoint ReferencePoint;

    if (BML_f_Abs( f_C0 ) > CURVATURE_USE_CIRCLE_EQUATION){
        f_Radius = 1.0f/(f_C0);
        /* Object Transform to Moment Pole Coordinates */    
        f_Y-=f_Radius;
        f_R = SQRT_( SQR(f_X) + SQR(f_Y));

        /* Check for divison by zero */
        /* Check for divison by zero */
        if (BML_f_IsZero(f_R)){
            /* Distances are zero */
        } else {
            /* NormVec to Course always pointing to the left side of course */
            if(f_C0 > 0.0f){
                f_NormVecX = -f_X/f_R;
                f_NormVecY = -f_Y/f_R;
                f_DistToCourse = (BML_f_Abs(f_Radius)-f_R);	
                f_DistOnCourse = f_Radius * (C_HALFPI + ATAN2_(f_Y, f_X));
            } else {
                f_NormVecX = f_X/f_R;
                f_NormVecY = f_Y/f_R;
                f_DistToCourse = -(BML_f_Abs(f_Radius)-f_R);		
                f_DistOnCourse = f_Radius * (ATAN2_(f_Y, f_X) - C_HALFPI);
            }
            /* DistCourse (fRadius-fR) positive when object left of course; negative when object right of course*/
            f_RefCourseDistX =  f_X - (f_NormVecX * f_DistToCourse);
            f_RefCourseDistY = (f_Y - (f_NormVecY * f_DistToCourse) ) + f_Radius;
        }
    } else {
        /* use old parabolic approximation for wide curves and distance in Y- Direction*/
        f_RefCourseDistX    = f_X;
        // FIXME: check the usage of f_X in calculation of f_RefCourseDistY
        f_RefCourseDistY    = SQR( f_X ) * (f_C0) * (0.5f);
        f_DistToCourse = (f_Y - f_RefCourseDistY);
        /*instead of integral 0 to fX of function SQRT(1+(fC0*x)^2) dx*/
        f_DistOnCourse = f_X;
    }

    ReferencePoint.f_X              = f_RefCourseDistX;
    ReferencePoint.f_Y              = f_RefCourseDistY;
    ReferencePoint.f_DistToTraj     = f_DistToCourse;
    ReferencePoint.f_DistOnTraj     = f_DistOnCourse;

    return ReferencePoint;    
}